@@ -10,7 +10,6 @@ from plone.app.layout.navigation.root import getNavigationRoot
 from plone.app.layout.navigation.root import getNavigationRootObject
 from plone.i18n.interfaces import ILanguageSchema
 from plone.memoize.view import memoize
-from plone.memoize.view import memoize_contextless
 from plone.protect.utils import addTokenToUrl
 from plone.registry.interfaces import IRegistry
 from Products.CMFCore.utils import getToolByName
@@ -243,7 +242,6 @@ class GlobalSectionsViewlet(ViewletBase):
     _subtree_markup_wrapper = u'<ul class="has_subtree dropdown">{out}</ul>'
 
     @property
-    @memoize_contextless
     def settings(self):
         registry = getUtility(IRegistry)
         settings = registry.forInterface(INavigationSchema, prefix="plone")
@@ -260,6 +258,7 @@ class GlobalSectionsViewlet(ViewletBase):
         return getNavigationRoot(self.context)
 
     @property
+    @deprecate("This property will be removed in Plone 6")
     def navtree_depth(self):
         return self.settings.navigation_depth
 
@@ -271,15 +270,26 @@ class GlobalSectionsViewlet(ViewletBase):
             or self.language_settings.default_language
         )
 
+    @property
+    def types_using_view(self):
+        registry = getUtility(IRegistry)
+        types_using_view = registry.get("plone.types_use_view_action_in_listings", [])
+        return types_using_view
+
     @property
     @memoize
     def navtree(self):
         ret = defaultdict(list)
+        settings = self.settings
         navtree_path = self.navtree_path
-        for tab in self.portal_tabs:
+        portal_tabs = self.portal_tabs
+        for tab in portal_tabs:
             entry = tab.copy()
             entry.update(
-                {"path": "/".join((navtree_path, tab["id"])), "uid": tab["id"],}
+                {
+                    "path": "/".join((navtree_path, tab["id"])),
+                    "uid": tab["id"],
+                }
             )
             if "review_state" not in entry:
                 entry["review_state"] = None
@@ -292,43 +302,48 @@ class GlobalSectionsViewlet(ViewletBase):
                     entry["title"], domain="plone", context=self.request
                 )
 
-            entry["title"] = safe_unicode(entry["title"])
+            entry["title"] = escape(safe_unicode(entry["title"]))
+            if "name" in entry and entry["name"]:
+                entry["name"] = escape(safe_unicode(entry["name"]))
+            self.customize_tab(entry, tab)
             ret[navtree_path].append(entry)
 
-        if not self.settings.generate_tabs:
+        if not settings.generate_tabs:
             return ret
 
         query = {
-            "path": {"query": self.navtree_path, "depth": self.navtree_depth,},
-            "portal_type": {"query": self.settings.displayed_types},
+            "path": {
+                "query": self.navtree_path,
+                "depth": settings.navigation_depth,
+            },
+            "portal_type": {"query": settings.displayed_types},
             "Language": self.current_language,
-            "sort_on": self.settings.sort_tabs_on,
+            "sort_on": settings.sort_tabs_on,
             "is_default_page": False,
         }
 
-        if self.settings.sort_tabs_reversed:
+        if settings.sort_tabs_reversed:
             query["sort_order"] = "reverse"
 
-        if not self.settings.nonfolderish_tabs:
+        if not settings.nonfolderish_tabs:
             query["is_folderish"] = True
 
-        if self.settings.filter_on_workflow:
-            query["review_state"] = list(self.settings.workflow_states_to_show or ())
+        if settings.filter_on_workflow:
+            query["review_state"] = list(settings.workflow_states_to_show or ())
 
-        if not self.settings.show_excluded_items:
+        if not settings.show_excluded_items:
             query["exclude_from_nav"] = False
 
         context_path = "/".join(self.context.getPhysicalPath())
+        self.customize_query(query)
         portal_catalog = getToolByName(self.context, "portal_catalog")
         brains = portal_catalog.searchResults(**query)
 
-        registry = getUtility(IRegistry)
-        types_using_view = registry.get("plone.types_use_view_action_in_listings", [])
-
+        types_using_view = self.types_using_view
         for brain in brains:
             brain_path = brain.getPath()
             brain_parent_path = brain_path.rpartition("/")[0]
-            if brain_parent_path == navtree_path:
+            if portal_tabs and brain_parent_path == navtree_path:
                 # This should be already provided by the portal_tabs_view
                 continue
             if brain.exclude_from_nav and not context_path.startswith(brain_path):
@@ -342,15 +357,24 @@ class GlobalSectionsViewlet(ViewletBase):
                 "path": brain_path,
                 "uid": brain.UID,
                 "url": url,
-                "title": safe_unicode(brain.Title),
+                "title": escape(safe_unicode(brain.Title)),
                 "review_state": brain.review_state,
             }
             self.customize_entry(entry, brain)
             ret[brain_parent_path].append(entry)
+
         return ret
 
+    def customize_query(self, query):
+        """Helper to customize the catalog query."""
+        pass
+
+    def customize_tab(self, entry, tab):
+        """Helper to add custom entry keys/values."""
+        pass
+
     def customize_entry(self, entry, brain):
-        """a little helper to add custom entry keys/values."""
+        """Helper to add custom entry keys/values."""
         pass
 
     def render_item(self, item, path):
@@ -368,10 +392,6 @@ class GlobalSectionsViewlet(ViewletBase):
             item.update(
                 {"sub": sub, "opener": "", "aria_haspopup": "", "has_sub_class": "",}
             )
-        if "title" in item and item["title"]:
-            item["title"] = escape(item["title"])
-        if "name" in item and item["name"]:
-            item["name"] = escape(item["name"])
         return self._item_markup_template.format(**item)
 
     def build_tree(self, path, first_run=True):
